#!/usr/bin/perl -w

use strict;
use DBI;
use Data::Dumper;
use Getopt::Std;
use POSIX;
use FindBin qw($Bin);

my $DBDIR="";
my $HTMLDIR="";
{
    my @a=split("/",$Bin);
    pop(@a);
    $DBDIR=join("/",@a)."/db";
    $HTMLDIR=join("/",@a)."/html";
}

my $MAINTAINER="";
{
    my %opts=();
    getopts("m:",\%opts);
    $MAINTAINER=$opts{m} if (defined $opts{m});
}

my $dbh=DBI->connect("dbi:SQLite:dbname=$DBDIR/newports.db","","");

mkdir("$HTMLDIR/maintainers",0755) if (!-d "$HTMLDIR/maintainers");
mkdir("$HTMLDIR/ports",0755) if (!-d "$HTMLDIR/ports");

my $NOW=POSIX::strftime("%Y-%m-%d",localtime);

my @maintainers=();
{
    my $WHERE="";
    $WHERE="WHERE maintainer='$MAINTAINER'" if ($MAINTAINER);
    my $sth=$dbh->prepare("
	SELECT	distinct maintainer
	FROM	newports
	$WHERE
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	push(@maintainers,$a[0]);
    }
}

my %ports=();
my %directories=();
{
    my $sth=$dbh->prepare("
	SELECT	directory,currentversion,newversion,url,maintainer
	FROM	newports
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	$ports{$a[0]}{$a[2]}{$a[3]}=1;
	$directories{$a[0]}{maintainer}=$a[4];
	$directories{$a[0]}{curversion}=$a[1];
    }
}

my %score=();
if (!$MAINTAINER) {
    open(FNAME,">$HTMLDIR/maintainers-name.html");
    print FNAME "<pre>";
}
foreach my $maintainer (sort nocase_sort (@maintainers)) {
    my %ports=();
    my %urllist=();
    my %directorylist=();

    my $sth=$dbh->prepare("
	SELECT	distinct category,port
	FROM	newports
	WHERE	maintainer like ?
    ");
    $sth->execute($maintainer);
    while (my @a=$sth->fetchrow_array()) {
	$ports{$a[0]}{$a[1]}=1;
    }

    my %urls=();
    foreach my $category (sort(keys(%ports))) {
	foreach my $port (sort(keys(%{$ports{$category}}))) {

	    my $sth=$dbh->prepare("
		SELECT	distinct directory,currentversion,newversion,url
		FROM	newports
		WHERE	maintainer=?
		AND	port=?
		AND	category=?
		ORDER BY newversion,url
	    ");
	    $sth->execute($maintainer,$port,$category);
	    while (my @a=$sth->fetchrow_array()) {
		$urls{$a[0]}{$a[1]}{$a[2]}{$a[3]}=0;
		$directorylist{$a[0]}{port}=$port;
		$directorylist{$a[0]}{category}=$category;
	    }
	}
    }

    my $txt="";
    $txt="
Port                                           Current version New version URLs
---------------------------------------------- --------------- ----------- ----
";

    my $urls=0;
    my $numports=0;
    foreach my $directory (sort(keys(%urls))) {
	foreach my $curversion (sort(keys(%{$urls{$directory}}))) {
	    foreach my $newversion (sort(keys(%{$urls{$directory}{$curversion}}))) {
		my $cleandir=$directory;
		$cleandir=~s/\//::/;
		$txt.=sprintf("<a href=\"../ports/%s.html\">%s</a>%s %-15s %-11s [%d]\n",
			$cleandir,
			$directory,
			" "x(45-length($directory)),
			$curversion,
			$newversion,
			++$urls);
		$numports++;
		foreach my $url (sort(keys(%{$urls{$directory}{$curversion}{$newversion}}))) {
		    $urllist{$urls}{$url}=1;
		}
	    }
	}
    }

    $txt.="
URLS
";

    foreach my $c (sort numeric_sort (keys(%urllist))) {
	my $i=0;
	foreach my $url (sort(keys(%{$urllist{$c}}))) {
	    if ($i==0) {
		$txt.=sprintf("[%d]%s ",$c,$c<10?"  ":$c<100?" ":"");
	    } else {
		$txt.="      ";
	    }
	    $txt.=sprintf("%s\n",$url);
	    $i++;
	}
    }

    print "Checking $maintainer\n";
    open(FOUT,">$HTMLDIR/maintainers/$maintainer.html");
    print FOUT "<h1>$maintainer</h1><p>Last run: $NOW</p><pre>$txt</pre>\n";
    close(FOUT);
    if (!$MAINTAINER) {
	printf FNAME ("<a href=\"maintainers/$maintainer.html\">%s</a>%s%3d ports\n",$maintainer," "x(40-length($maintainer)),$numports);
    }
    $score{$numports}{$maintainer}=1;
}
if (!$MAINTAINER) {
    close(FNAME);
    open(FSCORE,">$HTMLDIR/maintainers-score.html");
    print FSCORE "<pre>";
    foreach my $score (sort numeric_sort_rev (keys(%score))) {
	foreach my $maintainer (sort nocase_sort (keys(%{$score{$score}}))) {
	    printf FSCORE ("<a href=\"maintainers/$maintainer.html\">%s</a>%s%3d ports\n",$maintainer," "x(40-length($maintainer)),$score);
	}
    }
    print FSCORE "</pre>";
    close(FSCORE);
}

if (!$MAINTAINER) {
    open(FNAME,">$HTMLDIR/ports-name.html");
    print FNAME "<h1>By ports' name</h1><p>Last run: $NOW</p><pre>";
}
foreach my $directory (sort nocase_sort (keys(%ports))) {
    my $txt="";
    foreach my $newversion (sort nocase_sort (keys(%{$ports{$directory}}))) {
	$txt="
Port                                           Current version New version Maintainer
---------------------------------------------- --------------- ----------- ------------------------
";
	$txt.=sprintf("%-46s %-15s %-11s <a href=\"../maintainers/%s.html\">%s</a>\n",
		$directory,
		$directories{$directory}{curversion},
		$newversion,
		$directories{$directory}{maintainer},
		$directories{$directory}{maintainer}
	);
	$txt.="\n";
	foreach my $url (sort nocase_sort (keys(%{$ports{$directory}{$newversion}}))) {
	    $txt.="$url\n";
	}
    }

    print "Checking $directory\n";
    my $cleandir=$directory;
    $cleandir=~s/\//::/;
    open(FOUT,">$HTMLDIR/ports/$cleandir.html");
    print FOUT "<h1>$directory</h1><p>Last run: $NOW</p><pre>$txt</pre>\n";
    close(FOUT);
    if (!$MAINTAINER) {
	printf FNAME ("<a href=\"ports/$cleandir.html\">%s</a>\n",$directory);
    }
}
if (!$MAINTAINER) {
    close(FNAME);
}

sub nocase_sort {
     return lc($a) cmp lc($b);
}
sub numeric_sort {
     return $a <=> $b;
}
sub numeric_sort_rev {
     return $b <=> $a;
}

{
    open(FOUT,">$HTMLDIR/ignored-hosts.html");
    print FOUT "<h1>Ignored hosts</h1><p>Last run: $NOW</p><pre>\n";
    my $sth=$dbh->prepare("
	SELECT	hostname,protocol,reason
	FROM	ignorehosts
	ORDER BY hostname,protocol
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	printf FOUT "%s://%s - %s\n",$a[1],$a[0],$a[2];
    }
    close(FOUT);
}

{
    open(FOUT,">$HTMLDIR/ignored-maintainers.html");
    print FOUT "<h1>Ignored maintainers</h1><p>Last run: $NOW</p><pre>\n";
    my $sth=$dbh->prepare("
	SELECT	maintainer
	FROM	ignoremaintainers
	ORDER BY maintainer
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	printf FOUT "%s\n",$a[0];
    }
    close(FOUT);
}

{
    open(FOUT,">$HTMLDIR/ignored-ports.html");
    print FOUT "<h1>Ignored ports</h1><p>Last run: $NOW</p><pre>\n";
    my $sth=$dbh->prepare("
	SELECT	category,port
	FROM	ignoreports
	ORDER BY category,port
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	printf FOUT "%s/%s\n",$a[0],$a[1];
    }
    close(FOUT);
}

{
    open(FOUT,">$HTMLDIR/restricted-ports.html");
    print FOUT "<h1>Restricted ports</h1><p>Last run: $NOW</p><pre>\n";
    my $sth=$dbh->prepare("
	SELECT	directory,restriction
	FROM	restrictports
	ORDER BY directory
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	printf FOUT "%s - %s\n",$a[0],$a[1];
    }
    close(FOUT);
}
