#!/usr/bin/perl -w

# $Id$

#
# Manage the restrictions of directories.
#

use strict;
use DBI;
use Data::Dumper;
use FindBin qw($Bin);
use Getopt::Std;

my $DBDIR="";
{
    my @a=split("/",$Bin);
    pop(@a);
    $DBDIR=join("/",@a)."/db";
}

my $dbh=DBI->connect("dbi:SQLite:dbname=$DBDIR/newports.db","","");

my $ALL=0;
my $DIR="";
my $ERASE=0;
my $RESTRICTION="";
{
    my %opts=();
    getopts("ad:er:",\%opts);
    $ALL=1 if (defined $opts{a});
    $DIR=$opts{d} if (defined $opts{d});
    $ERASE=1 if (defined $opts{e});
    $RESTRICTION=$opts{r} if (defined $opts{r});
}

if (!$ALL && !$DIR) {
    print <<EOF;
Usage: $0 [-ae] [-d directory] [-r newrestrictions]
	-a	- view all restrictions
	-d	- view directory (unless -r)
	-e	- erase restriction
	-r	- set new restriction

The restriction can be a single one (1.) or multiple seperated by spaces (1. minor-even).
EOF
    exit;
}

my %restrictions=();

{
    my $WHERE="";
    $WHERE="WHERE directory='$DIR'" if ($DIR);

    my $sth=$dbh->prepare("
	SELECT	directory,restriction
	FROM	restrictports
	$WHERE
    ");
    $sth->execute();
    while (my @a=$sth->fetchrow_array()) {
	$restrictions{$a[0]}{$a[1]}=1;
    }
}

{
    foreach my $d (sort(keys(%restrictions))) {
	foreach my $r (sort(keys(%{$restrictions{$d}}))) {
	    print "$d\t- $r\n";
	}
    }
}

if ($RESTRICTION || $ERASE) {
    if (defined $restrictions{$DIR}) {
	my $sth_select=$dbh->prepare("
	    DELETE FROM  restrictports
	    WHERE	directory=?
	");
	$sth_select->execute($DIR);
	print "Removed previous restriction for $DIR\n";
    }
}

if ($RESTRICTION) {
    my $sth_select=$dbh->prepare("
	INSERT INTO restrictports(directory,restriction)
	VALUES (?,?)
    ");
    my @as=split(" ",$RESTRICTION);
    foreach my $a (@as) {
	$sth_select->execute($DIR,$a);
	print "Added new restriction for $a\n";
    }
}

$dbh->disconnect;
